@@ -3,8 +3,8 @@ import urlparse, urllib, time
 from zope.interface import Interface
 from twisted.web import html, resource
 from buildbot.status import builder
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
-
+from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED, EXCEPTION
+from buildbot import version, util
 
 class ITopBox(Interface):
     """I represent a box in the top row of the waterfall display: the one
@@ -32,6 +32,7 @@ class IHTMLLog(Interface):
 css_classes = {SUCCESS: "success",
                WARNINGS: "warnings",
                FAILURE: "failure",
+               SKIPPED: "skipped",
                EXCEPTION: "exception",
                }
 
@@ -89,17 +90,12 @@ def make_force_build_form(forceURL, on_all=False):
                  "<input type='text' name='revision' />")
       + '<input type="submit" value="Force Build" /></form>\n')
 
-colormap = {
-    'green': '#72ff75',
-    }
 def td(text="", parms={}, **props):
     data = ""
     data += "  "
     #if not props.has_key("border"):
     #    props["border"] = 1
     props.update(parms)
-    if props.has_key("bgcolor"):
-        props["bgcolor"] = colormap.get(props["bgcolor"], props["bgcolor"])
     comment = props.get("comment", None)
     if comment:
         data += "<!-- %s -->" % comment
@@ -107,7 +103,7 @@ def td(text="", parms={}, **props):
     class_ = props.get('class_', None)
     if class_:
         props["class"] = class_
-    for prop in ("align", "bgcolor", "colspan", "rowspan", "border",
+    for prop in ("align", "colspan", "rowspan", "border",
                  "valign", "halign", "class"):
         p = props.get(prop, None)
         if p != None:
@@ -170,15 +166,19 @@ def path_to_step(request, stepstatus):
     return (path_to_build(request, stepstatus.getBuild()) +
             "/steps/%s" % urllib.quote(stepstatus.getName(), safe=''))
 
+def path_to_slave(request, slave):
+    return (path_to_root(request) +
+            "buildslaves/" +
+            urllib.quote(slave.getName(), safe=''))
+
 class Box:
     # a Box wraps an Event. The Box has HTML <td> parameters that Events
     # lack, and it has a base URL to which each File's name is relative.
     # Events don't know about HTML.
     spacer = False
-    def __init__(self, text=[], color=None, class_=None, urlbase=None,
+    def __init__(self, text=[], class_=None, urlbase=None,
                  **parms):
         self.text = text
-        self.color = color
         self.class_ = class_
         self.urlbase = urlbase
         self.show_idle = 0
@@ -195,7 +195,7 @@ class Box:
         text = self.text
         if not text and self.show_idle:
             text = ["[idle]"]
-        return td(text, props, bgcolor=self.color, class_=self.class_)
+        return td(text, props, class_=self.class_)
 
 
 class HtmlResource(resource.Resource):
@@ -258,11 +258,39 @@ class HtmlResource(resource.Resource):
         return request.site.buildbot_service.getControl()
 
     def getChangemaster(self, request):
-        return request.site.buildbot_service.parent.change_svc
+        return request.site.buildbot_service.getChangeSvc()
 
     def path_to_root(self, request):
         return path_to_root(request)
 
+    def footer(self, s, req):
+        # TODO: this stuff should be generated by a template of some sort
+        projectURL = s.getProjectURL()
+        projectName = s.getProjectName()
+        data = '<hr /><div class="footer">\n'
+
+        welcomeurl = self.path_to_root(req) + "index.html"
+        data += '[<a href="%s">welcome</a>]\n' % welcomeurl
+        data += "<br />\n"
+
+        data += '<a href="http://buildbot.sourceforge.net/">Buildbot</a>'
+        data += "-%s " % version
+        if projectName:
+            data += "working for the "
+            if projectURL:
+                data += "<a href=\"%s\">%s</a> project." % (projectURL,
+                                                            projectName)
+            else:
+                data += "%s project." % projectName
+        data += "<br />\n"
+        data += ("Page built: " +
+                 time.strftime("%a %d %b %Y %H:%M:%S",
+                               time.localtime(util.now()))
+                 + "\n")
+        data += '</div>\n'
+
+        return data
+
     def getTitle(self, request):
         return self.title
 
@@ -349,7 +377,7 @@ class OneLineMixin:
         except KeyError:
             rev = "??"
         rev = str(rev)
-        if len(rev) > 20:
+        if len(rev) > 40:
             rev = "version is too-long"
         root = self.path_to_root(req)
         css_class = css_classes.get(results, "")
@@ -358,10 +386,8 @@ class OneLineMixin:
                   'buildnum': build.getNumber(),
                   'results': css_class,
                   'text': " ".join(build.getText()),
-                  'buildurl': (root +
-                               "builders/%s/builds/%d" % (builder_name,
-                                                          build.getNumber())),
-                  'builderurl': (root + "builders/%s" % builder_name),
+                  'buildurl': path_to_build(req, build),
+                  'builderurl': path_to_builder(req, build.getBuilder()),
                   'rev': rev,
                   'time': time.strftime(self.LINE_TIME_FORMAT,
                                         time.localtime(build.getTimes()[0])),
