@@ -7,7 +7,7 @@ import re, urllib, time
 from twisted.python import log
 from buildbot import interfaces
 from buildbot.status.web.base import HtmlResource, make_row, \
-     make_force_build_form, OneLineMixin
+     make_force_build_form, OneLineMixin, path_to_build, path_to_slave, path_to_builder
 from buildbot.process.base import BuildRequest
 from buildbot.sourcestamp import SourceStamp
 
@@ -27,7 +27,7 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
 
     def build_line(self, build, req):
         buildnum = build.getNumber()
-        buildurl = req.childLink("builds/%d" % buildnum)
+        buildurl = path_to_build(req, build)
         data = '<a href="%s">#%d</a> ' % (buildurl, buildnum)
 
         when = build.getETA()
@@ -43,7 +43,7 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
             # TODO: is this necessarily the case?
 
         if self.builder_control is not None:
-            stopURL = urllib.quote(req.childLink("builds/%d/stop" % buildnum))
+            stopURL = path_to_build(req, build) + '/stop'
             data += '''
 <form action="%s" class="command stopbuild" style="display:inline">
   <input type="submit" value="Stop Build" />
@@ -92,7 +92,8 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
         data += "<h2>Buildslaves:</h2>\n"
         data += "<ol>\n"
         for slave in slaves:
-            data += "<li><b>%s</b>: " % html.escape(slave.getName())
+            slaveurl = path_to_slave(req, slave)
+            data += "<li><b><a href=\"%s\">%s</a></b>: " % (html.escape(slaveurl), html.escape(slave.getName()))
             if slave.isConnected():
                 data += "CONNECTED\n"
                 if slave.getAdmin():
@@ -106,7 +107,7 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
         data += "</ol>\n"
 
         if control is not None and connected_slaves:
-            forceURL = urllib.quote(req.childLink("force"))
+            forceURL = path_to_builder(req, b) + '/force'
             data += make_force_build_form(forceURL)
         elif control is not None:
             data += """
@@ -115,7 +116,7 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
             """
 
         if control is not None:
-            pingURL = urllib.quote(req.childLink("ping"))
+            pingURL = path_to_builder(req, b) + '/ping'
             data += """
             <form action="%s" class='command pingbuilder'>
             <p>To ping the buildslave(s), push the 'Ping' button</p>
@@ -124,9 +125,20 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
             </form>
             """ % pingURL
 
+        data += self.footer(status, req)
+
         return data
 
     def force(self, req):
+        """
+
+        Custom properties can be passed from the web form.  To do
+        this, subclass this class, overriding the force() method.  You
+        can then determine the properties (usually from form values,
+        by inspecting req.args), then pass them to this superclass
+        force method.
+        
+        """
         name = req.args.get("username", ["<unknown>"])[0]
         reason = req.args.get("comments", ["<no reason specified>"])[0]
         branch = req.args.get("branch", [""])[0]
@@ -159,19 +171,21 @@ class StatusResourceBuilder(HtmlResource, OneLineMixin):
         # button, use their name instead of None, so they'll be informed of
         # the results.
         s = SourceStamp(branch=branch, revision=revision)
-        req = BuildRequest(r, s, self.builder_status.getName())
+        req = BuildRequest(r, s, builderName=self.builder_status.getName())
         try:
             self.builder_control.requestBuildSoon(req)
         except interfaces.NoSlaveError:
             # TODO: tell the web user that their request could not be
             # honored
             pass
-        return Redirect("../../waterfall")
+        # send the user back to the builder page
+        return Redirect(".")
 
     def ping(self, req):
         log.msg("web ping of builder '%s'" % self.builder_status.getName())
         self.builder_control.ping() # TODO: there ought to be an ISlaveControl
-        return Redirect("../../waterfall")
+        # send the user back to the builder page
+        return Redirect(".")
 
     def getChild(self, path, req):
         if path == "force":
@@ -229,7 +243,8 @@ class StatusResourceAllBuilders(HtmlResource, OneLineMixin):
                 builder_control = c.getBuilder(bname)
             build = StatusResourceBuilder(builder_status, builder_control)
             build.force(req)
-        return Redirect("../../waterfall")
+        # back to the welcome page
+        return Redirect("../..")
 
     def stop(self, req):
         for bname in self.status.getBuilderNames():
@@ -252,7 +267,8 @@ class StatusResourceAllBuilders(HtmlResource, OneLineMixin):
                 build = StatusResourceBuild(build_status, build_control,
                                             builder_control)
                 build.stop(req)
-        return Redirect("../../waterfall")
+        # go back to the welcome page
+        return Redirect("../..")
 
 
 # /builders
@@ -272,9 +288,11 @@ class BuildersResource(HtmlResource):
         for bname in s.getBuilderNames():
             data += (' <li><a href="%s">%s</a></li>\n' %
                      (req.childLink(urllib.quote(bname, safe='')),
-                      urllib.quote(bname, safe='')))
+                      bname))
         data += "</ol>\n"
 
+        data += self.footer(s, req)
+
         return data
 
     def getChild(self, path, req):
