@@ -18,6 +18,8 @@ from buildbot.status import base
 from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
 from buildbot.scripts.runner import ForceOptions
 
+from string import join, capitalize, lower
+
 class UsageError(ValueError):
     def __init__(self, string = "Invalid usage", *more):
         ValueError.__init__(self, string, *more)
@@ -49,7 +51,7 @@ class IrcBuildRequest:
         self.parent.send(response)
         self.parent.send("I'll give a shout when the build finishes")
         d = s.waitUntilFinished()
-        d.addCallback(self.parent.buildFinished)
+        d.addCallback(self.parent.watchedBuildFinished)
 
 
 class Contact:
@@ -66,6 +68,9 @@ class Contact:
 
     def __init__(self, channel):
         self.channel = channel
+        self.notify_events = {}
+        self.subscribed = 0
+	self.add_notification_events(channel.notify_events)
 
     silly = {
         "What happen ?": "Somebody set up us the bomb.",
@@ -165,6 +170,80 @@ class Contact:
         self.emit_status(which)
     command_STATUS.usage = "status [<which>] - List status of a builder (or all builders)"
 
+    def validate_notification_event(self, event):
+        if not re.compile("^(started|finished|success|failure|exception|warnings|(success|warnings|exception|failure)To(Failure|Success|Warnings|Exception))$").match(event):
+            raise UsageError("try 'notify on|off <EVENT>'")
+
+    def list_notified_events(self):
+        self.send( "The following events are being notified: %r" % self.notify_events.keys() )
+
+    def notify_for(self, *events):
+        for event in events:
+            if self.notify_events.has_key(event):
+                return 1
+        return 0
+
+    def subscribe_to_build_events(self):
+        self.channel.status.subscribe(self)
+        self.subscribed = 1
+
+    def unsubscribe_from_build_events(self):
+        self.channel.status.unsubscribe(self)
+        self.subscribed = 0
+
+    def add_notification_events(self, events):
+        for event in events:
+            self.validate_notification_event(event)
+            self.notify_events[event] = 1
+
+            if not self.subscribed:
+                self.subscribe_to_build_events()
+
+    def remove_notification_events(self, events):
+        for event in events:
+            self.validate_notification_event(event)
+            del self.notify_events[event]
+
+            if len(self.notify_events) == 0 and self.subscribed:
+                self.unsubscribe_from_build_events()
+
+    def remove_all_notification_events(self):
+        self.notify_events = {}
+
+        if self.subscribed:
+            self.unsubscribe_from_build_events()
+
+    def command_NOTIFY(self, args, who):
+        args = args.split()
+
+        if not args:
+            raise UsageError("try 'notify on|off|list <EVENT>'")
+        action = args.pop(0)
+        events = args
+
+        if action == "on":
+            if not events: events = ('started','finished')
+            self.add_notification_events(events)
+
+            self.list_notified_events()
+
+        elif action == "off":
+            if events:
+                self.remove_notification_events(events)
+            else:
+                self.remove_all_notification_events()
+
+            self.list_notified_events()
+
+        elif action == "list":
+            self.list_notified_events()
+            return
+
+        else:
+            raise UsageError("try 'notify on|off <EVENT>'")
+
+    command_NOTIFY.usage = "notify on|off|list [<EVENT>] ... - Notify me about build events.  event should be one or more of: 'started', 'finished', 'failure', 'success', 'exception' or 'xToY' (where x and Y are one of success, warnings, failure, exception, but Y is capitalized)"
+
     def command_WATCH(self, args, who):
         args = args.split()
         if len(args) != 1:
@@ -178,7 +257,7 @@ class Contact:
         for build in builds:
             assert not build.isFinished()
             d = build.waitUntilFinished()
-            d.addCallback(self.buildFinished)
+            d.addCallback(self.watchedBuildFinished)
             r = "watching build %s #%d until it finishes" \
                 % (which, build.getNumber())
             eta = build.getETA()
@@ -188,7 +267,94 @@ class Contact:
             self.send(r)
     command_WATCH.usage = "watch <which> - announce the completion of an active build"
 
-    def buildFinished(self, b):
+    def buildsetSubmitted(self, buildset):
+        log.msg('[Contact] Buildset %s added' % (buildset))
+
+    def builderAdded(self, builderName, builder):
+        log.msg('[Contact] Builder %s added' % (builder))
+        builder.subscribe(self)
+
+    def builderChangedState(self, builderName, state):
+        log.msg('[Contact] Builder %s changed state to %s' % (builderName, state))
+
+    def requestSubmitted(self, brstatus):
+        log.msg('[Contact] BuildRequest for %s submiitted to Builder %s' % 
+            (brstatus.getSourceStamp(), brstatus.builderName))
+
+    def builderRemoved(self, builderName):
+        log.msg('[Contact] Builder %s removed' % (builderName))
+
+    def buildStarted(self, builderName, build):
+        builder = build.getBuilder()
+        log.msg('[Contact] Builder %r in category %s started' % (builder, builder.category))
+
+        # only notify about builders we are interested in
+
+        if (self.channel.categories != None and
+           builder.category not in self.channel.categories):
+            log.msg('Not notifying for a build in the wrong category')
+            return
+
+        if not self.notify_for('started'):
+            log.msg('Not notifying for a build when started-notification disabled')
+            return
+
+        r = "build #%d of %s started" % \
+           (build.getNumber(),
+             builder.getName())
+
+        r += " including [" + ", ".join(map(lambda c: repr(c.revision), build.getChanges())) + "]"
+
+        self.send(r)
+
+    def buildFinished(self, builderName, build, results):
+        builder = build.getBuilder()
+
+        results_descriptions = {
+            SUCCESS: "Success",
+            WARNINGS: "Warnings",
+            FAILURE: "Failure",
+            EXCEPTION: "Exception",
+            }
+
+        # only notify about builders we are interested in
+        log.msg('[Contact] builder %r in category %s finished' % (builder, builder.category))
+
+        if self.notify_for('started'):
+            return
+
+        if (self.channel.categories != None and
+            builder.category not in self.channel.categories):
+            return
+
+        results = build.getResults()
+
+        r = "build #%d of %s is complete: %s" % \
+            (build.getNumber(),
+             builder.getName(),
+             results_descriptions.get(results, "??"))
+        r += " [%s]" % " ".join(build.getText())
+        buildurl = self.channel.status.getURLForThing(build)
+        if buildurl:
+            r += "  Build details are at %s" % buildurl
+
+        if self.notify_for('finished') or self.notify_for(lower(results_descriptions.get(results))):
+            self.send(r)
+            return
+
+        prevBuild = build.getPreviousBuild()
+        if prevBuild:
+            prevResult = prevBuild.getResults()
+
+            required_notification_control_string = join((lower(results_descriptions.get(prevResult)), \
+                                                             'To', \
+                                                             capitalize(results_descriptions.get(results))), \
+                                                            '')
+
+            if (self.notify_for(required_notification_control_string)):
+                self.send(r)
+
+    def watchedBuildFinished(self, b):
         results = {SUCCESS: "Success",
                    WARNINGS: "Warnings",
                    FAILURE: "Failure",
@@ -300,8 +466,8 @@ class Contact:
             last = b.getLastFinishedBuild()
             if last:
                 start,finished = last.getTimes()
-                str += ", last build %s secs ago: %s" % \
-                       (int(util.now() - finished), " ".join(last.getText()))
+                str += ", last build %s ago: %s" % \
+                        (self.convertTime(int(util.now() - finished)), " ".join(last.getText()))
         if state == "building":
             t = []
             for build in builds:
@@ -323,7 +489,7 @@ class Contact:
             str = "(no builds run since last restart)"
         else:
             start,finish = last.getTimes()
-            str = "%s secs ago: " % (int(util.now() - finish))
+            str = "%s ago: " % (self.convertTime(int(util.now() - finish)))
             str += " ".join(last.getText())
         self.send("last build [%s]: %s" % (which, str))
 
@@ -425,9 +591,23 @@ class IRCContact(Contact):
     # userJoined(self, user, channel)
 
     def send(self, message):
-        self.channel.msg(self.dest, message)
+        self.channel.msg(self.dest, message.encode("ascii", "replace"))
     def act(self, action):
-        self.channel.me(self.dest, action)
+        self.channel.me(self.dest, action.encode("ascii", "replace"))
+
+    def command_JOIN(self, args, who):
+        args = args.split()
+        to_join = args[0]
+        self.channel.join(to_join)
+        self.send("Joined %s" % to_join)
+    command_JOIN.usage = "join channel - Join another channel"
+
+    def command_LEAVE(self, args, who):
+        args = args.split()
+        to_leave = args[0]
+        self.send("Buildbot has been told to leave %s" % to_leave)
+        self.channel.part(to_leave)
+    command_LEAVE.usage = "leave channel - Leave a channel"
 
 
     def handleMessage(self, message, who):
@@ -485,7 +665,7 @@ class IrcStatusBot(irc.IRCClient):
     """
     implements(IChannel)
 
-    def __init__(self, nickname, password, channels, status, categories):
+    def __init__(self, nickname, password, channels, status, categories, notify_events):
         """
         @type  nickname: string
         @param nickname: the nickname by which this bot should be known
@@ -502,6 +682,7 @@ class IrcStatusBot(irc.IRCClient):
         self.password = password
         self.status = status
         self.categories = categories
+        self.notify_events = notify_events
         self.counter = 0
         self.hasQuit = 0
         self.contacts = {}
@@ -541,7 +722,7 @@ class IrcStatusBot(irc.IRCClient):
         # else it's a broadcast message, maybe for us, maybe not. 'channel'
         # is '#twisted' or the like.
         contact = self.getContact(channel)
-        if message.startswith("%s:" % self.nickname):
+        if message.startswith("%s:" % self.nickname) or message.startswith("%s," % self.nickname):
             message = message[len("%s:" % self.nickname):]
             contact.handleMessage(message, user)
         # to track users comings and goings, add code here
@@ -596,13 +777,14 @@ class IrcStatusFactory(ThrottledClientFactory):
     shuttingDown = False
     p = None
 
-    def __init__(self, nickname, password, channels, categories):
+    def __init__(self, nickname, password, channels, categories, notify_events):
         #ThrottledClientFactory.__init__(self) # doesn't exist
         self.status = None
         self.nickname = nickname
         self.password = password
         self.channels = channels
         self.categories = categories
+	self.notify_events = notify_events
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -617,7 +799,7 @@ class IrcStatusFactory(ThrottledClientFactory):
     def buildProtocol(self, address):
         p = self.protocol(self.nickname, self.password,
                           self.channels, self.status,
-                          self.categories)
+                          self.categories, self.notify_events)
         p.factory = self
         p.status = self.status
         p.control = self.control
@@ -650,7 +832,7 @@ class IRC(base.StatusReceiverMultiService):
                      "categories"]
 
     def __init__(self, host, nick, channels, port=6667, allowForce=True,
-                 categories=None, password=None):
+                 categories=None, password=None, notify_events={}):
         base.StatusReceiverMultiService.__init__(self)
 
         assert allowForce in (True, False) # TODO: implement others
@@ -663,10 +845,11 @@ class IRC(base.StatusReceiverMultiService):
         self.password = password
         self.allowForce = allowForce
         self.categories = categories
+	self.notify_events = notify_events
 
         # need to stash the factory so we can give it the status object
         self.f = IrcStatusFactory(self.nick, self.password,
-                                  self.channels, self.categories)
+                                  self.channels, self.categories, self.notify_events)
 
         c = internet.TCPClient(host, port, self.f)
         c.setServiceParent(self)
