@@ -18,6 +18,7 @@ class Source(LoggingBuildStep):
 
     # if the checkout fails, there's no point in doing anything else
     haltOnFailure = True
+    flunkOnFailure = True
     notReally = False
 
     branch = None # the default branch, should be set in __init__
@@ -47,8 +48,8 @@ class Source(LoggingBuildStep):
              should be maintained in a separate directory (called the
              'copydir'), using checkout or update as necessary. For each
              build, a new workdir is created with a copy of the source
-             tree (rm -rf workdir; cp -r copydir workdir). This doubles
-             the disk space required, but keeps the bandwidth low
+             tree (rm -rf workdir; cp -R -P -p copydir workdir). This
+             doubles the disk space required, but keeps the bandwidth low
              (update instead of a full checkout). A full 'clean' build
              is performed each time.  This avoids any generated-file
              build problems, but is still occasionally vulnerable to
@@ -159,7 +160,6 @@ class Source(LoggingBuildStep):
     def start(self):
         if self.notReally:
             log.msg("faking %s checkout/update" % self.name)
-            self.step_status.setColor("green")
             self.step_status.setText(["fake", self.name, "successful"])
             self.addCompleteLog("log",
                                 "Faked %s checkout/update 'successful'\n" \
@@ -184,10 +184,10 @@ class Source(LoggingBuildStep):
         self.startVC(branch, revision, patch)
 
     def commandComplete(self, cmd):
-        got_revision = None
         if cmd.updates.has_key("got_revision"):
-            got_revision = str(cmd.updates["got_revision"][-1])
-        self.setProperty("got_revision", got_revision)
+            got_revision = cmd.updates["got_revision"][-1]
+            if got_revision is not None:
+                self.setProperty("got_revision", str(got_revision), "Source")
 
 
 
@@ -212,7 +212,7 @@ class CVS(Source):
     # parsing each line. Might be handy to have a hook in LogFile that gets
     # called with each complete line.
 
-    def __init__(self, cvsroot, cvsmodule, 
+    def __init__(self, cvsroot, cvsmodule,
                  global_options=[], branch=None, checkoutDelay=None,
                  login=None,
                  **kwargs):
@@ -267,7 +267,7 @@ class CVS(Source):
                                read-only (I assume this means it won't
                                use locks to insure atomic access to the
                                ,v files)."""
-                               
+
         self.checkoutDelay = checkoutDelay
         self.branch = branch
 
@@ -351,7 +351,7 @@ class SVN(Source):
     name = 'svn'
 
     def __init__(self, svnurl=None, baseURL=None, defaultBranch=None,
-                 directory=None, **kwargs):
+                 directory=None, username=None, password=None, **kwargs):
         """
         @type  svnurl: string
         @param svnurl: the URL which points to the Subversion server,
@@ -366,12 +366,15 @@ class SVN(Source):
                         which a branch name will be appended. It should
                         probably end in a slash. Use exactly one of
                         C{svnurl} and C{baseURL}.
-                         
+
         @param defaultBranch: if branches are enabled, this is the branch
                               to use if the Build does not specify one
                               explicitly. It will simply be appended
                               to C{baseURL} and the result handed to
                               the SVN command.
+
+        @param username: username to pass to svn's --username
+        @param password: username to pass to svn's --password
         """
 
         if not kwargs.has_key('workdir') and directory is not None:
@@ -382,12 +385,16 @@ class SVN(Source):
         self.svnurl = svnurl
         self.baseURL = baseURL
         self.branch = defaultBranch
+        self.username = username
+        self.password = password
 
         Source.__init__(self, **kwargs)
         self.addFactoryArguments(svnurl=svnurl,
                                  baseURL=baseURL,
                                  defaultBranch=defaultBranch,
                                  directory=directory,
+                                 username=username,
+                                 password=password,
                                  )
 
         if not svnurl and not baseURL:
@@ -395,7 +402,7 @@ class SVN(Source):
 
 
     def computeSourceRevision(self, changes):
-        if not changes:
+        if not changes or None in [c.revision for c in changes]:
             return None
         lastChange = max([int(c.revision) for c in changes])
         return lastChange
@@ -459,6 +466,16 @@ class SVN(Source):
         self.args['revision'] = revision
         self.args['patch'] = patch
 
+        if self.username is not None or self.password is not None:
+            if self.slaveVersionIsOlderThan("svn", "2.8"):
+                m = ("This buildslave (%s) does not support svn usernames "
+                     "and passwords.  "
+                     "Refusing to build. Please upgrade the buildslave to "
+                     "buildbot-0.7.10 or newer." % (self.build.slavename,))
+                raise BuildSlaveTooOldError(m)
+            if self.username is not None: self.args['username'] = self.username
+            if self.password is not None: self.args['password'] = self.password
+
         revstuff = []
         if branch is not None and branch != self.branch:
             revstuff.append("[branch]")
@@ -476,9 +493,9 @@ class SVN(Source):
 class Darcs(Source):
     """Check out a source tree from a Darcs repository at 'repourl'.
 
-    To the best of my knowledge, Darcs has no concept of file modes. This
-    means the eXecute-bit will be cleared on all source files. As a result,
-    you may need to invoke configuration scripts with something like:
+    Darcs has no concept of file modes. This means the eXecute-bit will be
+    cleared on all source files. As a result, you may need to invoke
+    configuration scripts with something like:
 
     C{s(step.Configure, command=['/bin/sh', './configure'])}
     """
@@ -499,7 +516,7 @@ class Darcs(Source):
                         which a branch name will be appended. It should
                         probably end in a slash. Use exactly one of
                         C{repourl} and C{baseURL}.
-                         
+
         @param defaultBranch: if branches are enabled, this is the branch
                               to use if the Build does not specify one
                               explicitly. It will simply be appended to
@@ -591,7 +608,9 @@ class Git(Source):
         return changes[-1].revision
 
     def startVC(self, branch, revision, patch):
-        self.args['branch'] = branch
+        if branch is not None:
+            self.args['branch'] = branch
+
         self.args['revision'] = revision
         self.args['patch'] = patch
         slavever = self.slaveVersion("git")
@@ -789,7 +808,7 @@ class Bzr(Source):
                         which a branch name will be appended. It should
                         probably end in a slash. Use exactly one of
                         C{repourl} and C{baseURL}.
-                         
+
         @param defaultBranch: if branches are enabled, this is the branch
                               to use if the Build does not specify one
                               explicitly. It will simply be appended to
@@ -844,33 +863,44 @@ class Mercurial(Source):
     name = "hg"
 
     def __init__(self, repourl=None, baseURL=None, defaultBranch=None,
-                 **kwargs):
+                 branchType='dirname', **kwargs):
         """
         @type  repourl: string
         @param repourl: the URL which points at the Mercurial repository.
-                        This is used as the default branch. Using C{repourl}
-                        does not enable builds of alternate branches: use
-                        C{baseURL} to enable this. Use either C{repourl} or
-                        C{baseURL}, not both.
+                        This uses the 'default' branch unless defaultBranch is
+                        specified below and the C{branchType} is set to
+                        'inrepo'.  It is an error to specify a branch without
+                        setting the C{branchType} to 'inrepo'.
 
-        @param baseURL: if branches are enabled, this is the base URL to
-                        which a branch name will be appended. It should
-                        probably end in a slash. Use exactly one of
-                        C{repourl} and C{baseURL}.
+        @param baseURL: if 'dirname' branches are enabled, this is the base URL
+                        to which a branch name will be appended. It should
+                        probably end in a slash.  Use exactly one of C{repourl}
+                        and C{baseURL}.
 
         @param defaultBranch: if branches are enabled, this is the branch
                               to use if the Build does not specify one
-                              explicitly. It will simply be appended to
-                              C{baseURL} and the result handed to the
-                              'hg clone' command.
+                              explicitly.
+                              For 'dirname' branches, It will simply be
+                              appended to C{baseURL} and the result handed to
+                              the 'hg update' command.
+                              For 'inrepo' branches, this specifies the named
+                              revision to which the tree will update after a
+                              clone.
+
+        @param branchType: either 'dirname' or 'inrepo' depending on whether
+                           the branch name should be appended to the C{baseURL}
+                           or the branch is a mercurial named branch and can be
+                           found within the C{repourl}
         """
         self.repourl = repourl
         self.baseURL = baseURL
         self.branch = defaultBranch
+        self.branchType = branchType
         Source.__init__(self, **kwargs)
         self.addFactoryArguments(repourl=repourl,
                                  baseURL=baseURL,
                                  defaultBranch=defaultBranch,
+                                 branchType=branchType,
                                  )
         if (not repourl and not baseURL) or (repourl and baseURL):
             raise ValueError("you must provide exactly one of repourl and"
@@ -883,8 +913,11 @@ class Mercurial(Source):
                                         "about hg")
 
         if self.repourl:
-            assert not branch # we need baseURL= to use branches
+            # we need baseURL= to use dirname branches
+            assert self.branchType == 'inrepo' or not branch
             self.args['repourl'] = self.repourl
+            if branch:
+                self.args['branch'] = branch
         else:
             self.args['repourl'] = self.baseURL + branch
         self.args['revision'] = revision
