@@ -21,8 +21,8 @@ s = factory.s
 f1 = factory.QuickBuildFactory('fakerep', 'cvsmodule', configure=None)
 
 f2 = factory.BuildFactory([
-    s(dummy.Dummy, timeout=1),
-    s(dummy.RemoteDummy, timeout=2),
+    dummy.Dummy(timeout=1),
+    dummy.RemoteDummy(timeout=2),
     ])
 
 BuildmasterConfig = c = {}
@@ -317,7 +317,7 @@ class Disconnect(RunMixin, unittest.TestCase):
         # longer available and will be re-queued. Wait 5 seconds, then check
         # to make sure the build is still in the 'waiting for a slave' queue.
         self.control.getBuilder("dummy").original.START_BUILD_TIMEOUT = 1
-        req = BuildRequest("forced build", SourceStamp())
+        req = BuildRequest("forced build", SourceStamp(), "test_builder")
         self.failUnlessEqual(req.startCount, 0)
         self.control.getBuilder("dummy").requestBuild(req)
         # this should ping the slave, which doesn't respond, and then give up
@@ -638,10 +638,11 @@ c['schedulers'] = [
     Scheduler('triggerer', None, 0.1, ['triggerer']),
     Triggerable('triggeree', ['triggeree'])
 ]
-triggerer = factory.BuildFactory([
-    s(SetTestFlagStep, flagname='triggerer_started'),
-    s(Trigger, flunkOnFailure=True, @ARGS@),
-    s(SetTestFlagStep, flagname='triggerer_finished'),
+triggerer = factory.BuildFactory()
+triggerer.addSteps([
+    SetTestFlagStep(flagname='triggerer_started'),
+    Trigger(flunkOnFailure=True, @ARGS@),
+    SetTestFlagStep(flagname='triggerer_finished'),
     ])
 triggeree = factory.BuildFactory([
     s(SetTestFlagStep, flagname='triggeree_started'),
@@ -843,3 +844,356 @@ class TestFlag(RunMixin, TestFlagMixin, unittest.TestCase):
 # of the bug where I forgot to make Waterfall inherit from StatusReceiver
 # such that buildSetSubmitted failed.
 
+config_test_builder = config_base + """
+from buildbot.scheduler import Scheduler
+c['schedulers'] = [Scheduler('quick', 'dummy', 0.1, ['dummy']),
+                   Scheduler('quick2', 'dummy2', 0.1, ['dummy2']),
+                   Scheduler('quick3', 'dummy3', 0.1, ['dummy3'])]
+
+from buildbot.steps.shell import ShellCommand
+f3 = factory.BuildFactory([
+    s(ShellCommand, command="sleep 3", env={'blah':'blah'})
+    ])
+
+c['builders'] = [{'name': 'dummy', 'slavename': 'bot1', 'env': {'foo':'bar'},
+                  'builddir': 'dummy', 'factory': f3}]
+
+c['builders'].append({'name': 'dummy2', 'slavename': 'bot1',
+                       'env': {'blah':'bar'}, 'builddir': 'dummy2',
+                       'factory': f3})
+
+f4 = factory.BuildFactory([
+    s(ShellCommand, command="sleep 3")
+    ])
+
+c['builders'].append({'name': 'dummy3', 'slavename': 'bot1',
+                       'env': {'blah':'bar'}, 'builddir': 'dummy3',
+                       'factory': f4})
+"""
+
+class TestBuilder(RunMixin, unittest.TestCase):
+    def setUp(self):
+        RunMixin.setUp(self)
+        self.master.loadConfig(config_test_builder)
+        self.master.readConfig = True
+        self.master.startService()
+        self.connectSlave(builders=["dummy", "dummy2", "dummy3"])
+
+    def doBuilderEnvTest(self, branch, cb):
+        c = changes.Change("bob", ["Makefile", "foo/bar.c"], "changed",
+                           branch=branch)
+        self.master.change_svc.addChange(c)
+
+        d = defer.Deferred()
+        reactor.callLater(0.5, d.callback, None)
+        d.addCallback(cb)
+
+        return d
+
+    def testBuilderEnv(self):
+        return self.doBuilderEnvTest("dummy", self._testBuilderEnv1)
+
+    def _testBuilderEnv1(self, res):
+        b = self.master.botmaster.builders['dummy']
+        build = b.building[0]
+        s = build.currentStep
+        self.failUnless('foo' in s.cmd.args['env'])
+        self.failUnlessEqual('bar', s.cmd.args['env']['foo'])
+        self.failUnless('blah' in s.cmd.args['env'])
+        self.failUnlessEqual('blah', s.cmd.args['env']['blah'])
+
+    def testBuilderEnvOverride(self):
+        return self.doBuilderEnvTest("dummy2", self._testBuilderEnvOverride1)
+
+    def _testBuilderEnvOverride1(self, res):
+        b = self.master.botmaster.builders['dummy2']
+        build = b.building[0]
+        s = build.currentStep
+        self.failUnless('blah' in s.cmd.args['env'])
+        self.failUnlessEqual('blah', s.cmd.args['env']['blah'])
+
+    def testBuilderNoStepEnv(self):
+        return self.doBuilderEnvTest("dummy3", self._testBuilderNoStepEnv1)
+
+    def _testBuilderNoStepEnv1(self, res):
+        b = self.master.botmaster.builders['dummy3']
+        build = b.building[0]
+        s = build.currentStep
+        self.failUnless('blah' in s.cmd.args['env'])
+        self.failUnlessEqual('bar', s.cmd.args['env']['blah'])
+
+class SchedulerWatchers(RunMixin, TestFlagMixin, unittest.TestCase):
+    config_watchable = config_base + """
+from buildbot.scheduler import AnyBranchScheduler
+from buildbot.steps.dummy import Dummy
+from buildbot.test.runutils import setTestFlag, SetTestFlagStep
+s = AnyBranchScheduler(
+    name='abs',
+    branches=None,
+    treeStableTimer=0,
+    builderNames=['a', 'b'])
+c['schedulers'] = [ s ]
+
+# count the number of times a success watcher is called
+numCalls = [ 0 ]
+def watcher(ss):
+    numCalls[0] += 1
+    setTestFlag("numCalls", numCalls[0])
+s.subscribeToSuccessfulBuilds(watcher)
+
+f = factory.BuildFactory()
+f.addStep(Dummy(timeout=0))
+c['builders'] = [{'name': 'a', 'slavename': 'bot1',
+                  'builddir': 'a', 'factory': f},
+                 {'name': 'b', 'slavename': 'bot1',
+                  'builddir': 'b', 'factory': f}]
+"""
+
+    def testWatchers(self):
+        self.clearFlags()
+        m = self.master
+        m.loadConfig(self.config_watchable)
+        m.readConfig = True
+        m.startService()
+
+        c = changes.Change("bob", ["Makefile", "foo/bar.c"], "changed stuff")
+        m.change_svc.addChange(c)
+
+        d = self.connectSlave(builders=['a', 'b'])
+
+        def pause(res):
+            d = defer.Deferred()
+            reactor.callLater(1, d.callback, res)
+            return d
+        d.addCallback(pause)
+
+        def checkFn(res):
+            self.failUnlessEqual(self.getFlag('numCalls'), 1)
+        d.addCallback(checkFn)
+        return d
+
+config_priority = """
+from buildbot.process import factory
+from buildbot.steps import dummy
+from buildbot.buildslave import BuildSlave
+s = factory.s
+
+from buildbot.steps.shell import ShellCommand
+f1 = factory.BuildFactory([
+    s(ShellCommand, command="sleep 3", env={'blah':'blah'})
+    ])
+
+BuildmasterConfig = c = {}
+c['slaves'] = [BuildSlave('bot1', 'sekrit', max_builds=1)]
+c['schedulers'] = []
+c['builders'] = []
+c['builders'].append({'name':'quick1', 'slavename':'bot1', 'builddir': 'quickdir1', 'factory': f1})
+c['builders'].append({'name':'quick2', 'slavename':'bot1', 'builddir': 'quickdir2', 'factory': f1})
+c['slavePortnum'] = 0
+"""
+
+class BuildPrioritization(RunMixin, unittest.TestCase):
+    def rmtree(self, d):
+        rmtree(d)
+
+    def testPriority(self):
+        self.rmtree("basedir")
+        os.mkdir("basedir")
+        self.master.loadConfig(config_priority)
+        self.master.readConfig = True
+        self.master.startService()
+
+        d = self.connectSlave(builders=['quick1', 'quick2'])
+        d.addCallback(self._connected)
+
+        return d
+
+    def _connected(self, *args):
+        # Our fake source stamp
+        # we override canBeMergedWith so that our requests don't get merged together
+        ss = SourceStamp()
+        ss.canBeMergedWith = lambda x: False
+
+        # Send one request to tie up the slave before sending future requests
+        req0 = BuildRequest("reason", ss, "test_builder")
+        self.master.botmaster.builders['quick1'].submitBuildRequest(req0)
+
+        # Send 10 requests to alternating builders
+        # We fudge the submittedAt field after submitting since they're all
+        # getting submitted so close together according to time.time()
+        # and all we care about is what order they're run in.
+        reqs = []
+        self.finish_order = []
+        for i in range(10):
+            req = BuildRequest(str(i), ss, "test_builder")
+            j = i % 2 + 1
+            self.master.botmaster.builders['quick%i' % j].submitBuildRequest(req)
+            req.submittedAt = i
+            # Keep track of what order the builds finished in
+            def append(item, arg):
+                self.finish_order.append(item)
+            req.waitUntilFinished().addCallback(append, req)
+            reqs.append(req.waitUntilFinished())
+
+        dl = defer.DeferredList(reqs)
+        dl.addCallback(self._all_finished)
+
+        # After our first build finishes, we should wait for the rest to finish
+        d = req0.waitUntilFinished()
+        d.addCallback(lambda x: dl)
+        return d
+
+    def _all_finished(self, *args):
+        # The builds should have finished in proper order
+        self.failUnlessEqual([int(b.reason) for b in self.finish_order], range(10))
+
+# Test graceful shutdown when no builds are active, as well as
+# canStartBuild after graceful shutdown is initiated
+config_graceful_shutdown_idle = config_base
+class GracefulShutdownIdle(RunMixin, unittest.TestCase):
+    def testShutdown(self):
+        self.rmtree("basedir")
+        os.mkdir("basedir")
+        self.master.loadConfig(config_graceful_shutdown_idle)
+        self.master.readConfig = True
+        self.master.startService()
+        d = self.connectSlave(builders=['quick'])
+        d.addCallback(self._do_shutdown)
+        return d
+
+    def _do_shutdown(self, res):
+        bs = self.master.botmaster.builders['quick'].slaves[0]
+        # Check that the slave is accepting builds once it's connected
+        self.assertEquals(bs.slave.canStartBuild(), True)
+
+        # Monkeypatch the slave's shutdown routine since the real shutdown
+        # interrupts the test harness
+        self.did_shutdown = False
+        def _shutdown():
+            self.did_shutdown = True
+        bs.slave.shutdown = _shutdown
+
+        # Start a graceful shutdown
+        bs.slave.slave_status.setGraceful(True)
+        # Check that the slave isn't accepting builds any more
+        self.assertEquals(bs.slave.canStartBuild(), False)
+
+        # Wait a little bit and then check that we (pretended to) shut down
+        d = defer.Deferred()
+        d.addCallback(self._check_shutdown)
+        reactor.callLater(0.5, d.callback, None)
+        return d
+
+    def _check_shutdown(self, res):
+        self.assertEquals(self.did_shutdown, True)
+
+# Test graceful shutdown when two builds are active
+config_graceful_shutdown_busy = config_base + """
+from buildbot.buildslave import BuildSlave
+c['slaves'] = [ BuildSlave('bot1', 'sekrit', max_builds=2) ]
+
+from buildbot.scheduler import Scheduler
+c['schedulers'] = [Scheduler('dummy', None, 0.1, ['dummy', 'dummy2'])]
+
+c['builders'].append({'name': 'dummy', 'slavename': 'bot1',
+                      'builddir': 'dummy', 'factory': f2})
+c['builders'].append({'name': 'dummy2', 'slavename': 'bot1',
+                      'builddir': 'dummy2', 'factory': f2})
+"""
+class GracefulShutdownBusy(RunMixin, unittest.TestCase):
+    def testShutdown(self):
+        self.rmtree("basedir")
+        os.mkdir("basedir")
+        d = self.master.loadConfig(config_graceful_shutdown_busy)
+        d.addCallback(lambda res: self.master.startService())
+        d.addCallback(lambda res: self.connectSlave())
+
+        def _send(res):
+            # send a change. This will trigger both builders at the same
+            # time, but since they share a slave, the max_builds=1 setting
+            # will insure that only one of the two builds gets to run.
+            cm = self.master.change_svc
+            c = changes.Change("bob", ["Makefile", "foo/bar.c"],
+                               "changed stuff")
+            cm.addChange(c)
+        d.addCallback(_send)
+
+        def _delay(res):
+            d1 = defer.Deferred()
+            reactor.callLater(0.5, d1.callback, None)
+            # this test depends upon this 0.5s delay landing us in the middle
+            # of one of the builds.
+            return d1
+        d.addCallback(_delay)
+
+        # Start a graceful shutdown.  We should be in the middle of two builds
+        def _shutdown(res):
+            bs = self.master.botmaster.builders['dummy'].slaves[0]
+            # Monkeypatch the slave's shutdown routine since the real shutdown
+            # interrupts the test harness
+            self.did_shutdown = False
+            def _shutdown():
+                self.did_shutdown = True
+                return defer.succeed(None)
+            bs.slave.shutdown = _shutdown
+            # Start a graceful shutdown
+            bs.slave.slave_status.setGraceful(True)
+
+            builders = [ self.master.botmaster.builders[bn]
+                         for bn in ('dummy', 'dummy2') ]
+            for builder in builders:
+                self.failUnless(len(builder.slaves) == 1)
+            from buildbot.process.builder import BUILDING
+            building_bs = [ builder
+                            for builder in builders
+                            if builder.slaves[0].state == BUILDING ]
+            # assert that both builds are running right now.
+            self.failUnlessEqual(len(building_bs), 2)
+
+        d.addCallback(_shutdown)
+
+        # Wait a little bit again, and then make sure that we are still running
+        # the two builds, and haven't shutdown yet
+        d.addCallback(_delay)
+        def _check(res):
+            self.assertEquals(self.did_shutdown, False)
+            builders = [ self.master.botmaster.builders[bn]
+                         for bn in ('dummy', 'dummy2') ]
+            for builder in builders:
+                self.failUnless(len(builder.slaves) == 1)
+            from buildbot.process.builder import BUILDING
+            building_bs = [ builder
+                            for builder in builders
+                            if builder.slaves[0].state == BUILDING ]
+            # assert that both builds are running right now.
+            self.failUnlessEqual(len(building_bs), 2)
+        d.addCallback(_check)
+
+        # Wait for all the builds to finish
+        def _wait_finish(res):
+            builders = [ self.master.botmaster.builders[bn]
+                         for bn in ('dummy', 'dummy2') ]
+            builds = []
+            for builder in builders:
+                builds.append(builder.builder_status.currentBuilds[0].waitUntilFinished())
+            dl = defer.DeferredList(builds)
+            return dl
+        d.addCallback(_wait_finish)
+
+        # Wait a little bit after the builds finish, and then
+        # check that the slave has shutdown
+        d.addCallback(_delay)
+        def _check_shutdown(res):
+            # assert that we shutdown the slave
+            self.assertEquals(self.did_shutdown, True)
+            builders = [ self.master.botmaster.builders[bn]
+                         for bn in ('dummy', 'dummy2') ]
+            from buildbot.process.builder import BUILDING
+            building_bs = [ builder
+                            for builder in builders
+                            if builder.slaves[0].state == BUILDING ]
+            # assert that no builds are running right now.
+            self.failUnlessEqual(len(building_bs), 0)
+        d.addCallback(_check_shutdown)
+
+        return d
